#############################################################################
#
#	MacGofer Build file For MPW C/ASM
#
#	K. Hammond, January 31st. 1993
#
#############################################################################

#   File:       Gofer.make
#   Target:     Gofer
#   Created:    Thursday, January 2, 1992 4:06:24 pm

#   This makefile is set up for SADE debugging.
#   MPW C/Link V3.2 are required for this.
#   The MPW partition should be  1800K
#   NB: linking is MUCH faster with sym off

#   I have abandoned trying to squeeze all the global data into 32K:
#	Mark's changes in 2.28 made this unpleasant.  This adds about
#	60K to the size of the application, and heap requirements,
#	and may slow execution (since 32bit addresses rather than 16bit
#	offsets from A5 are now required to address global variables).


#	Use SYM = -sym on for debugging

SYM = -sym on

#	How to compile C programs

CC = C {SYM} -model far


#############################################################################
#
#	Standard Gofer Header files
#
#############################################################################


HDRS = prelude.h storage.h connect.h errors.h



#############################################################################
#
#	Standard Gofer Interpreter files
#
#############################################################################


builtin.c.o  	builtin.c prims.c {HDRS} mac_hdrs.h
	 {CC}  	builtin.c

cbuiltin.c.o  	cbuiltin.c prims.c {HDRS}
	 {CC}  	cbuiltin.c

compiler.c.o  	compiler.c {HDRS}
	 {CC}  	compiler.c

gofer.c.o  	gofer.c  machdep.c commonui.c {HDRS} command.h
	 {CC}  	gofer.c

gofc.c.o  	gofc.c  machdep.c commonui.c {HDRS} command.h
	 {CC}  	gofc.c

input.c.o  input.c  parser.c {HDRS} command.h
	 {CC}  input.c

machine.c.o 	machine.c {HDRS}
	 {CC}  	machine.c

cmachine.c.o 	cmachine.c {HDRS}
	 {CC}  	cmachine.c

output.c.o  	output.c {HDRS}
	 {CC}  	output.c

static.c.o  	static.c {HDRS}
	 {CC}  	static.c

storage.c.o  storage.c {HDRS}
	 {CC}  storage.c

type.c.o  type.c preds.c subst.c kind.c {HDRS}
	 {CC}  type.c


#############################################################################
#
#	 Gofer Runtime System
#
#############################################################################

runtime.c.o  runtime.c {HDRS}
	 {CC}  runtime.c


#############################################################################
#
#	Mac-specific files
#
#############################################################################

mac_dialogs.c.o  mac_dialogs.c mac.h
	 {CC}  mac_dialogs.c

mac_files.c.o  mac_files.c mac.h
	 {CC}  mac_files.c

mac_find.c.o  mac_find.c mac.h
	 {CC}  mac_find.c

mac_humayan_utils.c.o  mac_humayan_utils.c mac.h
	 {CC}  mac_humayan_utils.c

mac_io.c.o  mac_io.c mac.h
	 {CC}  mac_io.c

mac_main.c.o  mac_main.c mac.h
	 {CC}  mac_main.c

mac_menu.c.o  mac_menu.c mac.h
	 {CC}  mac_menu.c

mac_options.c.o  mac_options.c mac.h
	 {CC}  mac_options.c

mac_prefs.c.o  mac_prefs.c mac.h
	 {CC}  mac_prefs.c

mac_print.c.o  mac_print.c mac.h
	 {CC}  mac_print.c

mac_project.c.o  mac_project.c mac.h
	 {CC}  mac_project.c

mac_scrap.c.o  mac_scrap.c mac.h
	 {CC}  mac_scrap.c

mac_scroll.c.o  mac_scroll.c mac.h
	 {CC}  mac_scroll.c

mac_SFMultiGet.c.o  mac_SFMultiGet.c
	 {CC}  mac_SFMultiGet.c

mac_StrInit.c.o  mac_StrInit.c mac.h
	 {CC}  mac_StrInit.c

mac_undo.c.o  mac_undo.c mac.h
	 {CC}  mac_undo.c

mac_utils.c.o  mac_utils.c mac.h
	 {CC}  mac_utils.c

mac_windows.c.o  mac_windows.c mac.h
	 {CC}  mac_windows.c


#############################################################################
#
#	Assembler file for C monad
#
#############################################################################

mac_trap.a.o  mac_trap.a
	 Asm -case object -sym on mac_trap.a



#############################################################################
#
#	MacGofer Objects
#
#############################################################################

COMOBJS = compiler.c.o input.c.o output.c.o 
	  static.c.o storage.c.o type.c.o

IOBJECTS = builtin.c.o gofer.c.o machine.c.o {COMOBJS}
	  
COBJECTS = cbuiltin.c.o gofc.c.o cmachine.c.o {COMOBJS}

MACOBJS = mac_main.c.o mac_menu.c.o mac_utils.c.o mac_windows.c.o 
	  mac_print.c.o mac_files.c.o mac_scroll.c.o mac_project.c.o 
	  mac_find.c.o mac_dialogs.c.o mac_io.c.o mac_undo.c.o mac_prefs.c.o 
	  mac_options.c.o mac_trap.a.o 
	  mac_humayan_utils.c.o mac_scrap.c.o 
	  mac_StrInit.c.o mac_SFMultiGet.c.o
	  
ALLOBJS = {IOBJECTS} {MACOBJS}


#############################################################################
#
#	How to make an MPW tool
#
#############################################################################

Gofer   {IOBJECTS} Gofer.Make
	Link -w -ac 4 -ad 4 -srt -mf -model far 
		-t 'MPST' -c 'MPS ' 
		{IOBJECTS} 
		"{Libraries}"Stubs.o 
		"{Libraries}"Runtime.o 
		"{Libraries}"Interface.o 
		"{CLibraries}"StdCLib.o 
		"{CLibraries}"CSANELib.o 
		"{CLibraries}"Math.o 
		"{Libraries}"ToolLibs.o 
		-o Gofer

Gofc   {COBJECTS} Gofc.Make runtime.c.o
	Link -w -ac 4 -ad 4 -srt -mf -model far 
		-t 'MPST' -c 'MPS ' 
		{COBJECTS} 
		"{Libraries}"Stubs.o 
		"{Libraries}"Runtime.o 
		"{Libraries}"Interface.o 
		"{CLibraries}"StdCLib.o 
		"{CLibraries}"CSANELib.o 
		"{CLibraries}"Math.o 
		"{Libraries}"ToolLibs.o 
		-o Gofc

#############################################################################
#
#	Linking MacGofer under MPW 3.2.  You need MacGofer.r for this
#
#############################################################################

MacGofer  MacGofer.make MacGofer.r {ALLOBJS}
	Duplicate -y MacGofer.r MacGofer
	Link -w -ac 4 -ad 4 -srt -mf -model far 
	     -t APPL -c 'GFER' 
	     {SYM} 
	     -ra =resPreload 
		{ALLOBJS} 
		"{CLibraries}"CSANELib.o 
		"{CLibraries}"Math.o 
		"{CLibraries}"StdCLib.o 
		"{Libraries}"Runtime.o 
		"{Libraries}"Interface.o 
		"{Libraries}"ToolLibs.o 
		-o MacGofer

